fl

hex
{
DogPulseMonitor

     PropForth5.0
     
     Main     
     01/07/2012 23:12:10
     
     Power-switch   _power() ------ P5  
     Display-change _disp_sw() ---- P6
     
     Using 80MHz
}

5 wconstant _power
6 wconstant _disp_sw

\ constmt at 80MHz
E4E1C00 constant 3sec
5F5E1000 constant 20sec
{
\ constmt at 40MHz
7270E00 constant 3sec
2FAF0800 constant 20sec
}
wvariable time_out
wvariable sw

\ ( n1 -- n2 ) converting A/D-value to 0-7f(Y:128dots) and saving it to a/d_buf
\ n1:a/d_buf's index  
\ n2:level get level(devide by 512) from A/D-value and save to a/d_buf
: a/d_to_y
result L@ 9 rshift A0 swap - dup 
rot a/d_buf + C!           \ save value to a/d_buf 
;

\ Set x-y position
\ ( n1 n2 -- ) n1:Y n2:X 
: x_y_pos
2A write_command         \ set column address  X  
00 write_data
 write_data
2B write_command         \ set row address  Y  
00 write_data
 write_data
;

\ ( n -- ) n:remainder from "8 u/mod"
: plot 1 swap lshift a_byte_write ;

\ get sw-status for _disp_sw ( -- zero/not-zero)  zero:sw-On  not-zero:sw-Off
: sw_status ina COG@ 1 _disp_sw lshift and ;

\ Power-off when pushing sw during 3seconds and  switching mode when pushing sw during a moment.
\ ( -- 0/1/2 )  0:no sw-pushing
\               1:sw-pushing(sensing when sw released)
\               2:sw-pushing(sensing when sw-pushing exceed 3seconds) 
: read_sw
sw_status 0=                                      
if                                           \ sw pushed?
     A delms sw_status 0=                    \ wait chattering-time
     if
          cnt COG@                           \ load cnt1
          begin
               sw_status 0=                  \ check sw-status
               if
                    cnt COG@ over - 3sec >   \ sw pushed for 3seconds?
                    if
                         drop 2 1            \ return 2 (sw-push-time is 3seconds)
                    else
                         0
                    then
               else
                    drop 1 1                 \ return 1 (sw-push-time is less than 3seconds)
               then
          until
     else
          0                                  \ no sw-push
     then
else
     0                                       \ no sw-push
then
;          

\ ( -- )
: disp_sw
begin
     read_sw dup sw W!  
     begin
          dup   
          if
               sw W@ 0= if 3E8 delms else 0 then
          else
               drop 1
          then
     until
0 until
;


\ wait until AD-value > threshold  if success ( threshold -- threshold ) 
\                                  if time-out ( threshold -- 0 )
: wave_up
cnt COG@
begin
     swap dup result L@ <
     if
          nip 1                              \ success
     else
          swap cnt COG@ over - clkfreq >
          if
               2drop 0 time_out W! 1         \ time-out
          else
               0 
          then
     then
until
;

\ wait until AD-value < threshold  if success ( threshold -- cnt threshold )
\                                  if time-out ( threshold -- 0 )
: wave_down
cnt COG@
begin
     swap dup result L@ >
     if
          nip cnt COG@ swap 1                \ success
     else
          swap cnt COG@ over - clkfreq >
          if
               2drop 0 time_out W! 1         \ time-out
          else
               0 
          then
     then
until
;
     
: DogPulseMonitor
_power pinout _power pinhi                   \ set power_relay to On when poer-sw is pushed 

\      c" sigma_delta" 1 cogx                       \ start A/D-conversion
\      A delms                                      \ wait cog1

\      c" calc_threshold" 2 cogx                    \ start searching max\min
init_ST7735R                                 \ initialize LCD
1 rtc_busy W!
\      c" disp_sw" 3 cogx                           \ start switch-scan
_csx_out_l 

\ ======= Main loop ==================
begin
     Init_set                                          \ Inititial display setting
     0 LCD_dec                                         \ initial pulse value : 0
     
     \ Display Pulse-monitor
     begin
          1 time_out W!
          get_time                                     \ update time

          pulse_valid W@ dup 0= 
          if             \ Display "low"message if pulse is invalid 
               signal_low
          else           \ Erase "low"message
               18 01 7F 50 ram_addr 
               2C write_command
               F48 0 do 00 00 LCD_Datawrite loop      \ black
          then    
          if
               threshold L@ 
               wave_up time_out W@                     \ pulse-wave is up-direction?
               if                                     
                    \ Get first cnt [cnt1]
                    wave_down time_out W@                        \ pulse-wave is down-direction?
                    if                                  
                         wave_up time_out W@                     \ pulse-wave is up-direction?
                         if                              
                              \ Get second cnt [cnt2]
                              wave_down time_out W@              \ pulse-wave is down-direction?
                              if                          
                                   \ Get 60/cnt2-cnt1       Only display when 48 < pulse < 200
                                   drop swap - 20sec swap u/ 3 u* dup 30 C8 between                                                             
                                   if                       
                                        LCD_dec_erase
                                        C8 delms LCD_dec              \ 48 < pulse < 200
                                   else                     
                                        drop                          \ pulse <= 48   200 <= pulse
                                   then       
                              else
                                   drop
                              then
                         else
                              drop
                         then
                    then
               then
          then
          
          \ read sw-status
          sw W@ dup 2 = if drop _csx_out_h _res_out_h _res_out_l _res_out_h _power pinlo 1 then 
     until
     0 sw W!                       \ clear switch-sense
     
     \ Display endress Pulse-Wave
     FF FF screen_clear            \ paint screen by white
     cnt COG@ 15msec +             \ current time + 15msec
     \ X=128dots(time-axis)
     80 0 do
          i a/d_to_y 
          i 2+ x_y_pos                            \ set dot_position
          2C write_command 00 1F LCD_Datawrite    \ plot dot(blue-color)
          15msec waitcnt 
     loop
          
     begin
          \ Shift 127dots to left
          __scl __sda __d/cx a/d_buf a_data_shift          
          \ erase dot of right point
          7F a/d_buf + C@                         \ get last data
          81 x_y_pos
          2C write_command FF FF LCD_Datawrite    \ erase dot
          
          \ plot current dot to right point
          7F a/d_to_y                             \ get y-position from current a/d value        
          81 x_y_pos                              \ set dot_position
          2C write_command 00 1F LCD_Datawrite    \ plot dot(blue-color)

          15msec waitcnt
          \ read sw-status
          sw W@
     until
     drop
     0 sw W!                       \ clear switch-sense
          
     \ Display repeatable Pulse-Wave
     FF FF screen_clear            \ paint screen by white
     begin
          cnt COG@ 15msec +             \ current time + 10msec
          80 0 do
               i a/d_to_y 
               i 2+ x_y_pos                            \ set dot_position
               2C write_command 00 1F LCD_Datawrite    \ plot dot(blue-color)
               15msec waitcnt
               sw W@ if leave then 
          loop          
          drop
          __scl __sda __d/cx a/d_buf a_erase_dot
          64 delms
          \ read sw-status
          sw W@ if 1 else 0 then
     until
     0 sw W!                       \ clear switch-sense    
0 until
;


: onreset0 onreset DogPulseMonitor ;
: onreset1 onreset sigma_delta ;
: onreset2 onreset calc_threshold ;
: onreset3 onreset disp_sw ;
saveforth
reboot


 decimal
